==> n1ed-8.x-2.25.tar.gz: n1ed/src/Flmngr/FlmngrServer/fs/FMDiskFileSystem.php <==
<?php

namespace Drupal\n1ed\Flmngr\FlmngrServer\fs;

use Drupal\n1ed\Flmngr\FileUploaderServer\lib\action\resp\Message;
use Drupal\n1ed\Flmngr\FileUploaderServer\lib\MessageException;
use Drupal\n1ed\Flmngr\FlmngrServer\model\FMDir;
use Drupal\n1ed\Flmngr\FlmngrServer\model\FMFile;
use Drupal\n1ed\Flmngr\FlmngrServer\model\FMMessage;
use Drupal\n1ed\Flmngr\FlmngrServer\model\ImageInfo;
use Exception;

/**
 * Implements file system interface.
 */
class FMDiskFileSystem implements FMDiskFileSystemInterface {

  private $dirFiles;

  private $dirCache;

  /**
   * Creates an instance.
   */
  public function __construct($config) {
    $this->dirFiles = $config['dirFiles'];
    $this->dirCache = $config['dirCache'];
  }

  /**
   * {@inheritdoc}
   */
  public function getDirs() {
    $dirs = [];
    $fDir = $this->dirFiles;
    if (!file_exists($fDir) || !is_dir($fDir)) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_ROOT_DIR_DOES_NOT_EXIST));
    }

    $this->getDirsFill($dirs, $fDir, "");
    return $dirs;
  }

  /**
   * Fills direrctories list.
   */
  private function getDirsFill(&$dirs, $fDir, $path) {
    $files = scandir($fDir);

    if ($files === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_LIST_CHILDREN_IN_DIRECTORY));
    }

    $dirsCount = 0;
    $filesCount = 0;
    for ($i = 0; $i < count($files); $i++) {
      $file = $files[$i];
      if ($file === '.' || $file === '..') {
        continue;
      }
      if (is_file($fDir . '/' . $file)) {
        $filesCount++;
      }
      else {
        if (is_dir($fDir . '/' . $file)) {
          $dirsCount++;
        }
      }
    }

    $i = strrpos($fDir, '/');
    if ($i !== FALSE) {
      $dirName = substr($fDir, $i + 1);
    }
    else {
      $dirName = $fDir;
    }

    $dir = new FMDir($dirName, $path, $filesCount, $dirsCount);
    $dirs[] = $dir;

    for ($i = 0; $i < count($files); $i++) {
      if ($files[$i] !== '.' && $files[$i] !== '..') {
        if (is_dir($fDir . '/' . $files[$i])) {
          $this->getDirsFill($dirs, $fDir . '/' . $files[$i], $path . (strlen($path) > 0 ? "/" : "") . $dirName);
        }
      }
    }
  }

  /**
   * Gets relative path.
   */
  private function getRelativePath($path) {
    if (strpos($path, "..") !== FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_DIR_NAME_CONTAINS_INVALID_SYMBOLS));
    }

    $rootDirName = $this->getRootDirName();
    if (strpos($path, "/" . $rootDirName) !== 0) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_DIR_NAME_INCORRECT_ROOT));
    }

    return substr($path, strlen("/" . $rootDirName));
  }

  /**
   * Gets absolute path.
   */
  private function getAbsolutePath($path) {
    return $this->dirFiles . $this->getRelativePath($path);
  }

  /**
   * {@inheritdoc}
   */
  public function deleteDir($dirPath) {
    $fullPath = $this->getAbsolutePath($dirPath);
    $res = rmdir($fullPath);
    if ($res === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_DELETE_DIRECTORY));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function createDir($dirPath, $name) {
    if (strpos($name, "..") !== FALSE || strpos($name, "/") !== FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_DIR_NAME_CONTAINS_INVALID_SYMBOLS));
    }

    $fullPath = $this->getAbsolutePath($dirPath) . "/" . $name;
    $res = mkdir($fullPath, 0777, TRUE);
    if ($res === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_CREATE_DIRECTORY));
    }
  }

  /**
   * Renames file or directory.
   */
  private function renameFileOrDir($path, $newName) {

    if (strpos($newName, "..") !== FALSE || strpos($newName, "/") !== FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_DIR_NAME_CONTAINS_INVALID_SYMBOLS));
    }

    $fullPath = $this->getAbsolutePath($path);

    $i = strrpos($fullPath, "/");
    $fullPathDst = substr($fullPath, 0, $i + 1) . $newName;
    if (is_file($fullPathDst)) {
      throw new MessageException(Message::createMessage(Message::FILE_ALREADY_EXISTS, $newName));
    }

    $res = rename($fullPath, $fullPathDst);
    if ($res === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_RENAME));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function renameFile($filePath, $newName) {
    $this->renameFileOrDir($filePath, $newName);
  }

  /**
   * {@inheritdoc}
   */
  public function renameDir($dirPath, $newName) {
    $this->renameFileOrDir($dirPath, $newName);
  }

  /**
   * {@inheritdoc}
   */
  public function getFiles($dirPath) {

    $fullPath = $this->getAbsolutePath($dirPath);

    if (!is_dir($fullPath)) {
      throw new MessageException(Message::createMessage(Message::DIR_DOES_NOT_EXIST, $dirPath));
    }

    $fFiles = scandir($fullPath);
    if ($fFiles === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_DIR_CANNOT_BE_READ));
    }

    $files = [];
    for ($i = 0; $i < count($fFiles); $i++) {
      $fFile = $fFiles[$i];
      $fileFullPath = $fullPath . '/' . $fFile;
      if (is_file($fileFullPath)) {
        try {
          $imageInfo = $this->getImageInfo($fileFullPath);
        }
        catch (Exception $e) {
          $imageInfo = new ImageInfo();
          $imageInfo->width = NULL;
          $imageInfo->height = NULL;
        }
        $file = new FMFile($dirPath, $fFile, filesize($fileFullPath), filemtime($fileFullPath), $imageInfo);
        $files[] = $file;
      }
    }

    return $files;
  }

  /**
   * Gets image info.
   */
  private static function getImageInfo($file) {

    $size = getimagesize($file);
    if ($size === FALSE) {
      throw new MessageException(Message::createMessage(Message::IMAGE_PROCESS_ERROR));
    }

    $imageInfo = new ImageInfo();
    $imageInfo->width = $size[0];
    $imageInfo->height = $size[1];
    return $imageInfo;
  }

  /**
   * Gets root directory name.
   */
  private function getRootDirName() {
    $i = strrpos($this->dirFiles, "/");
    if ($i === FALSE) {
      return $this->dirFiles;
    }
    return substr($this->dirFiles, $i + 1);
  }

  /**
   * Resizes an image.
   */
  public function resizeImage($img, $width, $height) {
    // TODO:
  }

  /**
   * {@inheritdoc}
   */
  public function deleteFiles($filesPaths) {
    for ($i = 0; $i < count($filesPaths); $i++) {
      $fullPath = $this->getAbsolutePath($filesPaths[$i]);
      $res = is_dir($fullPath) ? rmdir($fullPath) : unlink($fullPath);
      if ($res === FALSE) {
        throw new MessageException(Message::createMessage(Message::UNABLE_TO_DELETE_FILE, $filesPaths[$i]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function copyFiles($filesPaths, $newPath) {
    for ($i = 0; $i < count($filesPaths); $i++) {
      $fullPathSrc = $this->getAbsolutePath($filesPaths[$i]);

      $index = strrpos($fullPathSrc, "/");
      $name = $index === FALSE ? $fullPathSrc : substr($fullPathSrc, $index + 1);
      $fullPathDst = $this->getAbsolutePath($newPath) . "/" . $name;

      $res = copy($fullPathSrc, $fullPathDst);
      if ($res === FALSE) {
        throw new MessageException(FMMessage::createMessage(FMMessage::FM_ERROR_ON_COPYING_FILES));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function moveFiles($filesPaths, $newPath) {
    for ($i = 0; $i < count($filesPaths); $i++) {
      $fullPathSrc = $this->getAbsolutePath($filesPaths[$i]);

      $index = strrpos($fullPathSrc, "/");
      $name = $index === FALSE ? $fullPathSrc : substr($fullPathSrc, $index + 1);
      $fullPathDst = $this->getAbsolutePath($newPath) . "/" . $name;

      $res = rename($fullPathSrc, $fullPathDst);
      if ($res === FALSE) {
        throw new MessageException(FMMessage::createMessage(FMMessage::FM_ERROR_ON_MOVING_FILES));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function moveDir($dirPath, $newPath) {
    $fullPathSrc = $this->getAbsolutePath($dirPath);

    $index = strrpos($fullPathSrc, "/");
    $name = $index === FALSE ? $fullPathSrc : substr($fullPathSrc, $index + 1);
    $fullPathDst = $this->getAbsolutePath($newPath) . "/" . $name;

    $res = rename($fullPathSrc, $fullPathDst);
    if ($res === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_ERROR_ON_MOVING_FILES));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function copyDir($dirPath, $newPath) {
    $fullPathSrc = $this->getAbsolutePath($dirPath);

    $index = strrpos($fullPathSrc, "/");
    $name = $index === FALSE ? $fullPathSrc : substr($fullPathSrc, $index + 1);
    $fullPathDst = $this->getAbsolutePath($newPath) . "/" . $name;

    $res = $this->copyDirRecurse($fullPathSrc, $fullPathDst);
    if ($res === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_ERROR_ON_MOVING_FILES));
    }
  }

  /**
   * Copies a directory recursive function.
   */
  private function copyDirRecurse($src, $dst) {
    $dir = opendir($src);
    mkdir($dst);
    while (FALSE !== ($file = readdir($dir))) {
      if (($file != '.') && ($file != '..')) {
        if (is_dir($src . '/' . $file)) {
          $res = $this->copyDirRecurse($src . '/' . $file, $dst . '/' . $file);
          if ($res === FALSE) {
            return FALSE;
          }
        }
        else {
          $res = copy($src . '/' . $file, $dst . '/' . $file);
          if ($res === FALSE) {
            return FALSE;
          }
        }
      }
    }
    closedir($dir);
    return TRUE;
  }

  /**
   * Checkes string ends with.
   */
  private static function endsWith($str, $ends) {
    return substr($str, -strlen($ends)) === $ends;
  }

  /**
   * Gets MIME type by extension.
   */
  private static function getMimeType($filePath) {
    $mimeType = NULL;
    if (FMDiskFileSystem::endsWith($filePath, '.png')) {
      $mimeType = "image/png";
    }
    if (FMDiskFileSystem::endsWith($filePath, '.gif')) {
      $mimeType = "image/gif";
    }
    if (FMDiskFileSystem::endsWith($filePath, '.bmp')) {
      $mimeType = "image/bmp";
    }
    if (FMDiskFileSystem::endsWith($filePath, '.jpg') || FMDiskFileSystem::endsWith($filePath, '.jpeg')) {
      $mimeType = "image/jpeg";
    }
    return $mimeType;
  }

  /**
   * {@inheritdoc}
   */
  public function getImagePreview($filePath, $width, $height) {

    $fullPath = $this->getAbsolutePath($filePath);
    $hash = md5($filePath . $width . $height . filesize($fullPath) . filemtime($fullPath));

    if (!file_exists($this->dirCache)) {
      if (!mkdir($this->dirCache)) {
        throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_CREATE_DIRECTORY));
      }
    }

    $fileCachedPath = $this->dirCache . '/' . $hash . '.png';
    if (!file_exists($fileCachedPath)) {

      $image = NULL;
      switch (FMDiskFileSystem::getMimeType($fullPath)) {
        case 'image/gif':
          $image = @imagecreatefromgif($fullPath);
          break;

        case 'image/jpeg':
          $image = @imagecreatefromjpeg($fullPath);
          break;

        case 'image/png':
          $image = @imagecreatefrompng($fullPath);
          break;

        case 'image/bmp':
          $image = @imagecreatefromwbmp($fullPath);
          break;
      }

      // Somewhy it can not read ONLY SOME JPEG files,
      // we've caught it on Windows + IIS + PHP
      // Solution from here: https://github.com/libgd/libgd/issues/206
      if (!$image) {
        $image = imagecreatefromstring(file_get_contents($fullPath));
      }

      if (!$image) {
        throw new MessageException(Message::createMessage(Message::IMAGE_PROCESS_ERROR));
      }
      imagesavealpha($image, TRUE);

      $imageInfo = FMDiskFileSystem::getImageInfo($fullPath);

      // Ratio thumb.
      $ratio_thumb = $width / $height;
      $xx = $imageInfo->width;
      $yy = $imageInfo->height;
      // Ratio original.
      $ratio_original = $xx / $yy;
      if ($ratio_original >= $ratio_thumb) {
        $yo = $yy;
        $xo = ceil(($yo * $width) / $height);
        $xo_ini = ceil(($xx - $xo) / 2);
        $xy_ini = 0;
      }
      else {
        $xo = $xx;
        $yo = ceil(($xo * $height) / $width);
        $xy_ini = ceil(($yy - $yo) / 2);
        $xo_ini = 0;
      }

      $resizedImage = imagecreatetruecolor($width, $height);
      imagecopyresampled($resizedImage, $image, 0, 0, $xo_ini, $xy_ini, $width, $height, $xo, $yo);

      if (imagepng($resizedImage, $fileCachedPath) === FALSE) {
        throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_WRITE_PREVIEW_IN_CACHE_DIR, $fileCachedPath));
      }
    }

    $f = fopen($fileCachedPath, 'rb');
    if ($f) {
      return ["image/png", $f];
    }
    throw new MessageException(FMMessage::createMessage(FMMessage::FM_FILE_DOES_NOT_EXIST));
  }

  /**
   * {@inheritdoc}
   */
  public function getImageOriginal($filePath) {

    $mimeType = FMDiskFileSystem::getMimeType($filePath);
    if ($mimeType == NULL) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_FILE_IS_NOT_IMAGE));
    }

    $fullPath = $this->getAbsolutePath($filePath);

    if (file_exists($fullPath)) {
      $f = fopen($fullPath, 'rb');
      if ($f) {
        return [$mimeType, $f];
      }
    }
    throw new MessageException(FMMessage::createMessage(FMMessage::FM_FILE_DOES_NOT_EXIST));
  }

  /**
   * {@inheritdoc}
   */
  public function getDirZipArchive($dirPath, $out) {
    // TODO: Implement getDirZipArchive() method.
  }

}

==> n1ed-8.x-2.26.tar.gz: n1ed/src/Flmngr/FlmngrServer/fs/FMDiskFileSystem.php <==
<?php

namespace Drupal\n1ed\Flmngr\FlmngrServer\fs;

use Drupal\n1ed\Flmngr\FileUploaderServer\lib\action\resp\Message;
use Drupal\n1ed\Flmngr\FileUploaderServer\lib\MessageException;
use Drupal\n1ed\Flmngr\FlmngrServer\model\FMDir;
use Drupal\n1ed\Flmngr\FlmngrServer\model\FMFile;
use Drupal\n1ed\Flmngr\FlmngrServer\model\FMMessage;
use Drupal\n1ed\Flmngr\FlmngrServer\model\ImageInfo;
use Exception;

/**
 * Implements file system interface.
 */
class FMDiskFileSystem implements FMDiskFileSystemInterface {

  private $dirFiles;

  private $dirCache;

  /**
   * Creates an instance.
   */
  public function __construct($config) {
    $this->dirFiles = $config['dirFiles'];
    $this->dirCache = $config['dirCache'];
  }

  /**
   * {@inheritdoc}
   */
  public function getDirs() {
    $dirs = [];
    $fDir = $this->dirFiles;
    if (!file_exists($fDir) || !is_dir($fDir)) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_ROOT_DIR_DOES_NOT_EXIST));
    }

    $this->getDirsFill($dirs, $fDir, "");
    return $dirs;
  }

  /**
   * Fills direrctories list.
   */
  private function getDirsFill(&$dirs, $fDir, $path) {
    $files = scandir($fDir);

    if ($files === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_LIST_CHILDREN_IN_DIRECTORY));
    }

    $dirsCount = 0;
    $filesCount = 0;
    for ($i = 0; $i < count($files); $i++) {
      $file = $files[$i];
      if ($file === '.' || $file === '..') {
        continue;
      }
      if (is_file($fDir . '/' . $file)) {
        $filesCount++;
      }
      else {
        if (is_dir($fDir . '/' . $file)) {
          $dirsCount++;
        }
      }
    }

    $i = strrpos($fDir, '/');
    if ($i !== FALSE) {
      $dirName = substr($fDir, $i + 1);
    }
    else {
      $dirName = $fDir;
    }

    $dir = new FMDir($dirName, $path, $filesCount, $dirsCount);
    $dirs[] = $dir;

    for ($i = 0; $i < count($files); $i++) {
      if ($files[$i] !== '.' && $files[$i] !== '..') {
        if (is_dir($fDir . '/' . $files[$i])) {
          $this->getDirsFill($dirs, $fDir . '/' . $files[$i], $path . (strlen($path) > 0 ? "/" : "") . $dirName);
        }
      }
    }
  }

  /**
   * Gets relative path.
   */
  private function getRelativePath($path) {
    if (strpos($path, "..") !== FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_DIR_NAME_CONTAINS_INVALID_SYMBOLS));
    }

    $rootDirName = $this->getRootDirName();
    if (strpos($path, "/" . $rootDirName) !== 0) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_DIR_NAME_INCORRECT_ROOT));
    }

    return substr($path, strlen("/" . $rootDirName));
  }

  /**
   * Gets absolute path.
   */
  private function getAbsolutePath($path) {
    return $this->dirFiles . $this->getRelativePath($path);
  }

  /**
   * {@inheritdoc}
   */
  public function deleteDir($dirPath) {
    $fullPath = $this->getAbsolutePath($dirPath);
    $res = rmdir($fullPath);
    if ($res === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_DELETE_DIRECTORY));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function createDir($dirPath, $name) {
    if (strpos($name, "..") !== FALSE || strpos($name, "/") !== FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_DIR_NAME_CONTAINS_INVALID_SYMBOLS));
    }

    $fullPath = $this->getAbsolutePath($dirPath) . "/" . $name;
    $res = mkdir($fullPath, 0777, TRUE);
    if ($res === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_CREATE_DIRECTORY));
    }
  }

  /**
   * Renames file or directory.
   */
  private function renameFileOrDir($path, $newName) {

    if (strpos($newName, "..") !== FALSE || strpos($newName, "/") !== FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_DIR_NAME_CONTAINS_INVALID_SYMBOLS));
    }

    $fullPath = $this->getAbsolutePath($path);

    $i = strrpos($fullPath, "/");
    $fullPathDst = substr($fullPath, 0, $i + 1) . $newName;
    if (is_file($fullPathDst)) {
      throw new MessageException(Message::createMessage(Message::FILE_ALREADY_EXISTS, $newName));
    }

    $res = rename($fullPath, $fullPathDst);
    if ($res === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_RENAME));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function renameFile($filePath, $newName) {
    $this->renameFileOrDir($filePath, $newName);
  }

  /**
   * {@inheritdoc}
   */
  public function renameDir($dirPath, $newName) {
    $this->renameFileOrDir($dirPath, $newName);
  }

  /**
   * {@inheritdoc}
   */
  public function getFiles($dirPath) {

    $fullPath = $this->getAbsolutePath($dirPath);

    if (!is_dir($fullPath)) {
      throw new MessageException(Message::createMessage(Message::DIR_DOES_NOT_EXIST, $dirPath));
    }

    $fFiles = scandir($fullPath);
    if ($fFiles === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_DIR_CANNOT_BE_READ));
    }

    $files = [];
    for ($i = 0; $i < count($fFiles); $i++) {
      $fFile = $fFiles[$i];
      $fileFullPath = $fullPath . '/' . $fFile;
      if (is_file($fileFullPath)) {
        try {
          $imageInfo = $this->getImageInfo($fileFullPath);
        }
        catch (Exception $e) {
          $imageInfo = new ImageInfo();
          $imageInfo->width = NULL;
          $imageInfo->height = NULL;
        }
        $file = new FMFile($dirPath, $fFile, filesize($fileFullPath), filemtime($fileFullPath), $imageInfo);

        list($previewFormat, $previewFile) = $this->getImagePreview($fileFullPath, 159, 139);
        $previewData = '';
        while (!feof($previewFile)) {
          $previewData .= fread($previewFile, 8192);
        }
        fclose($previewFile);
        $file->preview = "data:" . $previewFormat . ";base64," . base64_encode($previewData);

        $files[] = $file;
      }
    }

    return $files;
  }

  /**
   * Gets image info.
   */
  private static function getImageInfo($file) {

    $size = getimagesize($file);
    if ($size === FALSE) {
      throw new MessageException(Message::createMessage(Message::IMAGE_PROCESS_ERROR));
    }

    $imageInfo = new ImageInfo();
    $imageInfo->width = $size[0];
    $imageInfo->height = $size[1];
    return $imageInfo;
  }

  /**
   * Gets root directory name.
   */
  private function getRootDirName() {
    $i = strrpos($this->dirFiles, "/");
    if ($i === FALSE) {
      return $this->dirFiles;
    }
    return substr($this->dirFiles, $i + 1);
  }

  /**
   * Resizes an image.
   */
  public function resizeImage($img, $width, $height) {
    // TODO:
  }

  /**
   * {@inheritdoc}
   */
  public function deleteFiles($filesPaths) {
    for ($i = 0; $i < count($filesPaths); $i++) {
      $fullPath = $this->getAbsolutePath($filesPaths[$i]);
      $res = is_dir($fullPath) ? rmdir($fullPath) : unlink($fullPath);
      if ($res === FALSE) {
        throw new MessageException(Message::createMessage(Message::UNABLE_TO_DELETE_FILE, $filesPaths[$i]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function copyFiles($filesPaths, $newPath) {
    for ($i = 0; $i < count($filesPaths); $i++) {
      $fullPathSrc = $this->getAbsolutePath($filesPaths[$i]);

      $index = strrpos($fullPathSrc, "/");
      $name = $index === FALSE ? $fullPathSrc : substr($fullPathSrc, $index + 1);
      $fullPathDst = $this->getAbsolutePath($newPath) . "/" . $name;

      $res = copy($fullPathSrc, $fullPathDst);
      if ($res === FALSE) {
        throw new MessageException(FMMessage::createMessage(FMMessage::FM_ERROR_ON_COPYING_FILES));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function moveFiles($filesPaths, $newPath) {
    for ($i = 0; $i < count($filesPaths); $i++) {
      $fullPathSrc = $this->getAbsolutePath($filesPaths[$i]);

      $index = strrpos($fullPathSrc, "/");
      $name = $index === FALSE ? $fullPathSrc : substr($fullPathSrc, $index + 1);
      $fullPathDst = $this->getAbsolutePath($newPath) . "/" . $name;

      $res = rename($fullPathSrc, $fullPathDst);
      if ($res === FALSE) {
        throw new MessageException(FMMessage::createMessage(FMMessage::FM_ERROR_ON_MOVING_FILES));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function moveDir($dirPath, $newPath) {
    $fullPathSrc = $this->getAbsolutePath($dirPath);

    $index = strrpos($fullPathSrc, "/");
    $name = $index === FALSE ? $fullPathSrc : substr($fullPathSrc, $index + 1);
    $fullPathDst = $this->getAbsolutePath($newPath) . "/" . $name;

    $res = rename($fullPathSrc, $fullPathDst);
    if ($res === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_ERROR_ON_MOVING_FILES));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function copyDir($dirPath, $newPath) {
    $fullPathSrc = $this->getAbsolutePath($dirPath);

    $index = strrpos($fullPathSrc, "/");
    $name = $index === FALSE ? $fullPathSrc : substr($fullPathSrc, $index + 1);
    $fullPathDst = $this->getAbsolutePath($newPath) . "/" . $name;

    $res = $this->copyDirRecurse($fullPathSrc, $fullPathDst);
    if ($res === FALSE) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_ERROR_ON_MOVING_FILES));
    }
  }

  /**
   * Copies a directory recursive function.
   */
  private function copyDirRecurse($src, $dst) {
    $dir = opendir($src);
    mkdir($dst);
    while (FALSE !== ($file = readdir($dir))) {
      if (($file != '.') && ($file != '..')) {
        if (is_dir($src . '/' . $file)) {
          $res = $this->copyDirRecurse($src . '/' . $file, $dst . '/' . $file);
          if ($res === FALSE) {
            return FALSE;
          }
        }
        else {
          $res = copy($src . '/' . $file, $dst . '/' . $file);
          if ($res === FALSE) {
            return FALSE;
          }
        }
      }
    }
    closedir($dir);
    return TRUE;
  }

  /**
   * Checkes string ends with.
   */
  private static function endsWith($str, $ends) {
    return substr($str, -strlen($ends)) === $ends;
  }

  /**
   * Gets MIME type by extension.
   */
  private static function getMimeType($filePath) {
    $mimeType = NULL;
    if (FMDiskFileSystem::endsWith($filePath, '.png')) {
      $mimeType = "image/png";
    }
    if (FMDiskFileSystem::endsWith($filePath, '.gif')) {
      $mimeType = "image/gif";
    }
    if (FMDiskFileSystem::endsWith($filePath, '.bmp')) {
      $mimeType = "image/bmp";
    }
    if (FMDiskFileSystem::endsWith($filePath, '.jpg') || FMDiskFileSystem::endsWith($filePath, '.jpeg')) {
      $mimeType = "image/jpeg";
    }
    return $mimeType;
  }

  /**
   * {@inheritdoc}
   */
  public function getImagePreview($fullPath, $width, $height) {

    $hash = md5($filePath . $width . $height . filesize($fullPath) . filemtime($fullPath));

    if (!file_exists($this->dirCache)) {
      if (!mkdir($this->dirCache)) {
        throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_CREATE_DIRECTORY));
      }
    }

    $fileCachedPath = $this->dirCache . '/' . $hash . '.png';
    if (!file_exists($fileCachedPath)) {

      $image = NULL;
      switch (FMDiskFileSystem::getMimeType($fullPath)) {
        case 'image/gif':
          $image = @imagecreatefromgif($fullPath);
          break;

        case 'image/jpeg':
          $image = @imagecreatefromjpeg($fullPath);
          break;

        case 'image/png':
          $image = @imagecreatefrompng($fullPath);
          break;

        case 'image/bmp':
          $image = @imagecreatefromwbmp($fullPath);
          break;
      }

      // Somewhy it can not read ONLY SOME JPEG files,
      // we've caught it on Windows + IIS + PHP
      // Solution from here: https://github.com/libgd/libgd/issues/206
      if (!$image) {
        $image = imagecreatefromstring(file_get_contents($fullPath));
      }

      if (!$image) {
        throw new MessageException(Message::createMessage(Message::IMAGE_PROCESS_ERROR));
      }
      imagesavealpha($image, TRUE);

      $imageInfo = FMDiskFileSystem::getImageInfo($fullPath);

      // Ratio thumb.
      $ratio_thumb = $width / $height;
      $xx = $imageInfo->width;
      $yy = $imageInfo->height;
      // Ratio original.
      $ratio_original = $xx / $yy;
      if ($ratio_original >= $ratio_thumb) {
        $yo = $yy;
        $xo = ceil(($yo * $width) / $height);
        $xo_ini = ceil(($xx - $xo) / 2);
        $xy_ini = 0;
      }
      else {
        $xo = $xx;
        $yo = ceil(($xo * $height) / $width);
        $xy_ini = ceil(($yy - $yo) / 2);
        $xo_ini = 0;
      }

      $resizedImage = imagecreatetruecolor($width, $height);
      imagecopyresampled($resizedImage, $image, 0, 0, $xo_ini, $xy_ini, $width, $height, $xo, $yo);

      if (imagepng($resizedImage, $fileCachedPath) === FALSE) {
        throw new MessageException(FMMessage::createMessage(FMMessage::FM_UNABLE_TO_WRITE_PREVIEW_IN_CACHE_DIR, $fileCachedPath));
      }
    }

    $f = fopen($fileCachedPath, 'rb');
    if ($f) {
      return ["image/png", $f];
    }
    throw new MessageException(FMMessage::createMessage(FMMessage::FM_FILE_DOES_NOT_EXIST));
  }

  /**
   * {@inheritdoc}
   */
  public function getImageOriginal($filePath) {

    $mimeType = FMDiskFileSystem::getMimeType($filePath);
    if ($mimeType == NULL) {
      throw new MessageException(FMMessage::createMessage(FMMessage::FM_FILE_IS_NOT_IMAGE));
    }

    $fullPath = $this->getAbsolutePath($filePath);

    if (file_exists($fullPath)) {
      $f = fopen($fullPath, 'rb');
      if ($f) {
        return [$mimeType, $f];
      }
    }
    throw new MessageException(FMMessage::createMessage(FMMessage::FM_FILE_DOES_NOT_EXIST));
  }

  /**
   * {@inheritdoc}
   */
  public function getDirZipArchive($dirPath, $out) {
    // TODO: Implement getDirZipArchive() method.
  }

}
